﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Xml.Linq;
using Wizard.UI;

namespace Footbreak.AddISOWizard
{
    public partial class SelectEpisodes : Wizard.UI.InternalWizardPage
    {
        private ISOData mISOData;

        public SelectEpisodes( ISOData aISOData )
        {
            mISOData = aISOData;
            InitializeComponent();
            return;
        }

        private void SelectEpisodes_SetActive( object sender, CancelEventArgs e )
        {
            SetWizardButtons( WizardButtons.Next | WizardButtons.Back );

            label1.Text = mISOData.ISOPath;
            label2.Text = mISOData.SeriesName;
            label3.Text = String.Format( "Season {0}", mISOData.SeasonNumber );

            foreach ( HandbrakeTitle lTitle in mISOData.HandbrakeTitles )
            {
                lTitle.EpisodeNumber = 0;
                lTitle.EpisodeTitle = "";
            }

            dataGridView1.DataSource = null;
            dataGridView1.Rows.Clear();
            dataGridView1.DataSource = mISOData.HandbrakeTitles;
            List<string> lDataSource = ( from s in mISOData.SeriesInformation.Descendants( "Episode" )
                                         where s.Descendants( "SeasonNumber" ).First().Value == mISOData.SeasonNumber.ToString()
                                         orderby Int32.Parse( s.Descendants( "EpisodeNumber" ).First().Value )
                                         select String.Format( "{0} - {1}",
                                                               s.Descendants( "EpisodeNumber" ).First().Value,
                                                               s.Descendants( "EpisodeName" ).First().Value ) ).ToList();
            
            lDataSource.Insert( 0, "" );
            lDataSource.Insert( 1, "Not Listed" );
            Episode.DataSource = lDataSource;
            return;
        }

        private void dataGridView1_CellEndEdit( object sender, DataGridViewCellEventArgs e )
        {
            if ( e.ColumnIndex == 2 )
            {
                HandbrakeTitle lHandbrakeTitle = ( HandbrakeTitle )dataGridView1.Rows[e.RowIndex].DataBoundItem;

                string lValue = (string)dataGridView1.Rows[e.RowIndex].Cells[e.ColumnIndex].Value;
                if ( lValue != null )
                {
                    string[] lData = Regex.Split( lValue, " - " );

                    if ( lData != null && lData.Length == 2 )
                    {
                        lHandbrakeTitle.EpisodeNumber = UInt32.Parse( lData[0] );
                        lHandbrakeTitle.EpisodeTitle = lData[1];
                    }
                    else
                    {
                        lHandbrakeTitle.EpisodeNumber = 0;
                        lHandbrakeTitle.EpisodeTitle = lValue;
                    }
                }
                else
                {
                    lHandbrakeTitle.EpisodeNumber = 0;
                    lHandbrakeTitle.EpisodeTitle = "";
                }
            }
            return;
        }
    }
}
