﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.IO;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Globalization;
using System.Threading;
using System.Web;
using System.Xml;
using System.Xml.Linq;
using Wizard.UI;

namespace Footbreak.AddISOWizard
{
    public partial class SelectSeries : Wizard.UI.InternalWizardPage
    {
        private ISOData mISOData;

        private struct SeriesData
        {
            public int ID { get; set; }
            public string Name { get; set; }
            public string Overview { get; set; }

            public override string ToString()
            {
                return Name;
            }
        }

        public SelectSeries( ISOData aISOData )
        {
            mISOData = aISOData;
            InitializeComponent();
            return;
        }

        private void SelectSeries_SetActive( object sender, CancelEventArgs e )
        {
            if ( mISOData.SeriesID > 0 )
            {
                SetWizardButtons( WizardButtons.Next | WizardButtons.Back );
            }
            else
            {
                SetWizardButtons( WizardButtons.Back );
            }

            if ( mISOData.SeriesName != "" )
            {
                textBox1.Text = mISOData.SeriesName;
            }
            else
            {
                string lSearch = System.IO.Path.GetFileNameWithoutExtension( mISOData.ISOPath );
                lSearch = Regex.Replace( lSearch, @"([SDY]\d|-|_|\.)", " " ).ToLower();
                lSearch = Regex.Replace( lSearch, @"\s+", " " );
                lSearch = Regex.Replace( lSearch, @"\s$", "" );
                lSearch = Regex.Replace( lSearch, @"^\s", "" );
                lSearch = Thread.CurrentThread.CurrentCulture.TextInfo.ToTitleCase( lSearch );
                textBox1.Text = lSearch;

                if ( mISOData.SeriesID == 0 )
                {
                    button1_Click( null, null );
                    if ( listBox1.Items.Count > 0 )
                    {
                        listBox1.SelectedIndex = 0;
                        //listBox1.Select();
                    }
                }
            }
            return;
        }

        private void button1_Click( object sender, EventArgs e )
        {
            // Clear existing data stuff
            mISOData.SeriesID = 0;
            mISOData.SeriesName = "";
            mISOData.SeriesInformation = null;
            mISOData.SeasonNumber = 0;
            listBox1.Items.Clear();
            textBox2.Text = "";
            SetWizardButtons( WizardButtons.Back );

            // Search for series
            string lRequestURI = String.Format( "http://www.thetvdb.com/api/GetSeries.php?seriesname={0}", HttpUtility.UrlEncode( textBox1.Text, Encoding.UTF8 ) );
            HttpWebRequest lHTPPRequest = ( HttpWebRequest )WebRequest.Create( lRequestURI );
            HttpWebResponse lHTTPResponse = ( HttpWebResponse )lHTPPRequest.GetResponse();
            Stream lResponseStream = lHTTPResponse.GetResponseStream();
            // Create a new XML document.
            XmlDocument lDoc = new XmlDocument();

            // Read the XML stream and dump it directly into the XmlDocument
            StreamReader lReader = new StreamReader( lResponseStream );
            lDoc.LoadXml( lReader.ReadToEnd() );
            lReader.Close();

            // Extracts the results from the XmlDocument
            XmlNodeList lXmlResults = lDoc.GetElementsByTagName( "Series" );
            foreach ( XmlNode lXmlResult in lXmlResults )
            {
                SeriesData lSeriesData = new SeriesData();

                foreach ( XmlNode lChildNode in lXmlResult.ChildNodes )
                {
                    if ( lChildNode.Name == "seriesid" )
                    {
                        int lID;
                        if ( Int32.TryParse( lChildNode.FirstChild.Value, out lID ) )
                        {
                            lSeriesData.ID = lID;
                        }
                    }
                    else if ( lChildNode.Name == "SeriesName" )
                    {
                        lSeriesData.Name = lChildNode.FirstChild.Value;
                    }
                    else if ( lChildNode.Name == "Overview" )
                    {
                        if ( lChildNode.FirstChild != null )
                        {
                            lSeriesData.Overview = lChildNode.FirstChild.Value;
                        }
                        else
                        {
                            lSeriesData.Overview = "Unknown";
                        }
                    }
                }

                if ( lSeriesData.ID != 0 && lSeriesData.Name != "" )
                {
                    listBox1.Items.Add( lSeriesData );
                }
            }
            if ( listBox1.Items.Count == 0 )
            {
                textBox2.Text = "No search results.";
            }
            return;
        }

        private void listBox1_SelectedValueChanged( object sender, EventArgs e )
        {
            SeriesData lSeriesData = (SeriesData)listBox1.SelectedItem;
            textBox2.Text = lSeriesData.Overview;
            mISOData.SeriesName = lSeriesData.Name;
            mISOData.SeriesID = lSeriesData.ID;
            SetWizardButtons( WizardButtons.Back | WizardButtons.Next );
            return;
        }

        private void SelectSeries_WizardNext( object sender, WizardPageEventArgs e )
        {
            if ( mISOData.SeriesInformation == null )
            {
                string lRequestURI = String.Format( "http://thetvdb.com/api/100C8353F30A9F43/series/{0}/all/en.xml", mISOData.SeriesID );
                mISOData.SeriesInformation = XElement.Load( lRequestURI );
            }
            return;
        }
    }
}
