using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using System.Diagnostics;
using System.Windows.Forms;

namespace Footbreak
{
    class HandbrakeCommandLine
    {
        private static Regex cPercentCompleteEncode = new Regex( @"^Encoding: task (\d+) of (\d+), (\d+)\." );
        private string mExePath;
        private bool mAbort = false;
        public event StatusDelegate mUpdateStatusEvent;
        public event PercentDelegate mUpdatePercentEvent;
        public event EventDelegate mCompletedEvent;
        public event AddTitleDelegate mAddTitleEvent;
        public event RemoveTitleDelegate mRemoveTitleEvent;

        public HandbrakeCommandLine( string aExePath )
        {
            mExePath = aExePath;
            return;
        }

        public bool Abort
        {
            get
            {
                return mAbort;
            }
            set
            {
                mAbort = value;
            }
        }

        public void getTitles( object aISOPaths )
        {
            mAbort = false;
            string[] lISOPaths = (string[])aISOPaths;
            uint lISOPathCount = 0;
            foreach ( string lISOPath in lISOPaths )
            {
                if ( mUpdateStatusEvent != null )
                {
                    mUpdateStatusEvent.Invoke( "Reading " + lISOPath );
                }
                getTitles( lISOPath );
                lISOPathCount += 1;
                if ( mUpdatePercentEvent != null )
                {
                    mUpdatePercentEvent.Invoke( ( int )( 100 * lISOPathCount / lISOPaths.Length ) );
                }
                if ( mAbort )
                {
                    break;
                }
            }
            mCompletedEvent.Invoke();
            return;
        }

        protected void getTitles( string aISOPath )
        {
            // Create the process
            string lArguments = "-i \"" + aISOPath + "\" -t0";
            Process lHandbrakeProcess = new Process();
            ProcessStartInfo lHandbrakeStartInfo = new ProcessStartInfo( mExePath, lArguments );
            lHandbrakeStartInfo.FileName = mExePath;
            lHandbrakeStartInfo.UseShellExecute = false;
            lHandbrakeStartInfo.RedirectStandardError = true;
            lHandbrakeStartInfo.CreateNoWindow = true;
            lHandbrakeProcess.StartInfo = lHandbrakeStartInfo;

            // Start the process and attach the output stream
            lHandbrakeProcess.Start();
            string lConsoleLinesString = lHandbrakeProcess.StandardError.ReadToEnd();
            lHandbrakeProcess.WaitForExit();
            
            // Extract titles
            List<HandbrakeTitle> lTitles = new List<HandbrakeTitle>();
            List<string> lTempTitleInfo = null;

            string[] lConsoleLines = lConsoleLinesString.Split( "\n".ToCharArray() );
            foreach ( string lConsoleLine in lConsoleLines )
            {
                if ( lConsoleLine.StartsWith( "+ title " ) )
                {
                    if ( lTempTitleInfo != null )
                    {
                        mAddTitleEvent.Invoke( new HandbrakeTitle( aISOPath, lTempTitleInfo ) );
                    }
                    lTempTitleInfo = new List<string>();
                    lTempTitleInfo.Add( lConsoleLine );
                }
                else if ( lConsoleLine.StartsWith( "HandBrake has exited" ) && lTempTitleInfo != null )
                {
                    mAddTitleEvent.Invoke( new HandbrakeTitle( aISOPath, lTempTitleInfo ) );
                }
                else if ( lTempTitleInfo != null )
                {
                    lTempTitleInfo.Add( lConsoleLine );
                }
                if ( mAbort )
                {
                    break;
                }
            }
            return;
        }

        public void convertTitles( object aData )
        {
            Pair<HandbrakeSettings, BindingSource> lThreadData = ( Pair<HandbrakeSettings, BindingSource> )aData;
            mAbort = false;
            int lTitleCount = 0;
            while ( lThreadData.Second.Count > 0 )
            {
                HandbrakeTitle lTitle = lThreadData.Second[0] as HandbrakeTitle;
                mRemoveTitleEvent.Invoke( lTitle );

                convertTitle( lThreadData.First, lTitle );
                lTitleCount += 1;
                mUpdatePercentEvent.Invoke( lTitleCount * 100 / lThreadData.Second.Count );
                if ( mAbort )
                {
                    break;
                }
            }
            mCompletedEvent.Invoke();
            return;
        }

        public void convertTitle( HandbrakeSettings aSettings, HandbrakeTitle aTitle )
        {
            // Determine output path
            string lOutputPath = aSettings.OutputPath;
            lOutputPath = lOutputPath.Replace( "{Series Name}", aTitle.SeriesName );
            lOutputPath = lOutputPath.Replace( "{Season}", aTitle.SeasonNumber.ToString() );
            lOutputPath = lOutputPath.Replace( "{Episode Number}", String.Format( "{0:d2}", aTitle.EpisodeNumber ) );
            lOutputPath = lOutputPath.Replace( "{Episode Title}", aTitle.EpisodeTitle );

            // Create directory
            Directory.CreateDirectory( Path.GetDirectoryName( lOutputPath ) );

            // Update status bar
            mUpdateStatusEvent.Invoke( "Encoding Title " + aTitle.Title + ", " + aTitle.ISOPath );

            // Create the argument list
            string lArguments = "-i \"" + aTitle.ISOPath + "\" -t " + aTitle.Title + " -o \"" +
                lOutputPath + "\" -e " + aSettings.VideoEncoder.ToString() + " -E " +
                aSettings.AudioEncoder.ToString() + " -b " +
                aSettings.VideoBitrate + " -B " + aSettings.AudioBitrate + " -R ";
            switch ( aSettings.AudioSampleRate )
	        {
		        case HandbrakeSettings.AudioSampleRateType.rate_22_05:
                    lArguments += "22.05";
                    break;
                case HandbrakeSettings.AudioSampleRateType.rate_24_00:
                    lArguments += "24";
                    break;
                case HandbrakeSettings.AudioSampleRateType.rate_32_00:
                    lArguments += "32";
                    break;
                case HandbrakeSettings.AudioSampleRateType.rate_44_10:
                    lArguments += "44.1";
                    break;
                case HandbrakeSettings.AudioSampleRateType.rate_48_00:
                    lArguments += "48";
                    break;
                default:
                    break;
	        }
            if ( aSettings.TwoPass )
            {
                lArguments += " -2 -T";
            }
            if ( aSettings.Deinterlace == HandbrakeSettings.DeinterlaceType.Fast )
            {
                lArguments += " --deinterlace=\"-1\"";
            }
            else if ( aSettings.Deinterlace == HandbrakeSettings.DeinterlaceType.Slow )
            {
                lArguments += " --deinterlace=\"0\"";
            }
            if ( aSettings.ChapterMarkers )
            {
                lArguments += " -m";
            }

            // Create the process
            Process lHandbrakeProcess = new Process();
            ProcessStartInfo lHandbrakeStartInfo = new ProcessStartInfo( mExePath, lArguments );
            lHandbrakeStartInfo.CreateNoWindow = false;
            lHandbrakeProcess.StartInfo = lHandbrakeStartInfo;

            // Start the process
            lHandbrakeProcess.Start();

            while ( lHandbrakeProcess.HasExited == false )
            {
                System.Threading.Thread.Sleep( 1000 );
                if ( mAbort )
                {
                    lHandbrakeProcess.Kill();
                    break;
                }
                /*string lLine = lHandbrakeProcess.StandardError.ReadLine();
                if ( lLine != null && cPercentCompleteEncode.IsMatch( lLine ) )
                {
                    Match lMatch = cPercentCompleteEncode.Match( lLine );
                    int lThisTask = Int32.Parse( lMatch.Groups[1].Value ) - 1;
                    int lTasksTotal = Int32.Parse( lMatch.Groups[2].Value );
                    int lThisTaskPercent = Int32.Parse( lMatch.Groups[3].Value );
                    int lThisEncodeBase = 100 * lThisTask / lTasksTotal;
                    int lThisEncodePercent = lThisEncodeBase + lThisTaskPercent / lTasksTotal;
                    int lTotalPercent = lBasePercent + lThisEncodePercent / aTotalCount;

                    
                }*/
            }
            return;
        }
    }
}
