using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace Footbreak
{
    public class HandbrakeTitle
    {
        private static Regex cTitleMatch = new Regex( @"^\+ title (\d+)" );
        private static Regex cDurationMatch = new Regex( @"^  \+ duration: ((\d\d):(\d\d):(\d\d))" );

        private bool mSelected;
        private readonly string mISOPath;
        private readonly uint mTitle = 0;
        private readonly string mLength;
        private uint mEpisodeNumber;
        private string mEpisodeTitle;

        public HandbrakeTitle( string aISOPath, List<string> aTitleInformation )
        {
            mISOPath = aISOPath;

            foreach ( string lLine in aTitleInformation )
            {
                // Match title
                if ( mTitle == 0 && cTitleMatch.IsMatch( lLine ) )
                {
                    Match lMatch = cTitleMatch.Match( lLine );
                    mTitle = UInt32.Parse( lMatch.Groups[1].Value );
                }

                // Match duration
                if ( mLength == null && cDurationMatch.IsMatch( lLine ) )
                {
                    Match lMatch = cDurationMatch.Match( lLine );
                    mLength = lMatch.Groups[1].Value;

                    uint lLengthInSeconds = UInt32.Parse( lMatch.Groups[2].Value ) * 60 * 60;
                    lLengthInSeconds += UInt32.Parse( lMatch.Groups[3].Value ) * 60;
                    lLengthInSeconds += UInt32.Parse( lMatch.Groups[4].Value );

                    /*mTargetSize = (uint)( lLengthInSeconds / 6.9 );
                    if ( mTargetSize > 175 && mTargetSize < 225 )
                    {
                        mTargetSize = 200;
                    }
                    else if ( mTargetSize > 325 && mTargetSize < 375 )
                    {
                        mTargetSize = 400;
                    }*/
                    if ( lLengthInSeconds > 20 * 60 )
                    {
                        mSelected = true;
                    }
                }
            }
            return;
        }

        public bool Selected
        {
            get
            {
                return mSelected;
            }
            set
            {
                mSelected = value;
            }
        }

        public string ISOPath
        {
            get
            {
                return mISOPath;
            }
        }

        public uint Title
        {
            get
            {
                return mTitle;
            }
        }

        public string Length
        {
            get
            {
                return mLength;
            }
        }

        public uint EpisodeNumber
        {
            get
            {
                return mEpisodeNumber;
            }
            set
            {
                mEpisodeNumber = value;
            }
        }
        public string EpisodeTitle
        {
            get
            {
                return mEpisodeTitle;
            }
            set
            {
                mEpisodeTitle = value;
            }
        }

        public string SeriesName { get; set; }
        public uint SeasonNumber { get; set; }
    }
}
